from __future__ import annotations

import asyncio
import base64
from datetime import datetime
from typing import TYPE_CHECKING, Any, Literal, cast

import jinja2
import yarl
from aiotieba.api.get_posts._classdef import Thread_p
from aiotieba.typing import Post, Thread

from ..client import Client
from ..models.dto import BaseUserDTO, CommentDTO, PostDTO, ThreadDTO, ThreadpDTO
from ..parser import convert_aiotieba_post, convert_aiotieba_thread, convert_aiotieba_threadp
from ..utils.logger import logger
from .config import RenderConfig
from .playwright_core import PlaywrightCore
from .style import get_font_style

if TYPE_CHECKING:
    from collections.abc import Sequence
    from pathlib import Path


def format_date(dt: datetime | int | float) -> str:
    if isinstance(dt, (int, float)):
        if dt > 1e11:
            dt = dt / 1000
        dt = datetime.fromtimestamp(dt)
    return dt.strftime("%Y-%m-%d %H:%M")


class Renderer:
    """
    渲染器，用于将贴子数据渲染为图像

    Args:
        client: 用于获取资源的客户端实例，若为 None 则创建新的 Client 实例
        config: 渲染配置，若为 None 则使用默认配置
        template_dir: 自定义模板目录，若为 None 则使用内置模板
    """

    def __init__(
        self,
        client: Client | None = None,
        config: RenderConfig | None = None,
        template_dir: str | Path | None = None,
    ) -> None:
        self.core = PlaywrightCore()

        if config is None:
            config = RenderConfig()
        self.config = config

        self.client = client or Client()
        self._own_client = client is None
        self._client_entered = False

        loader: jinja2.BaseLoader
        if template_dir:
            loader = jinja2.FileSystemLoader(str(template_dir))
        else:
            loader = jinja2.PackageLoader("tiebameow.renderer", "templates")

        self.env = jinja2.Environment(loader=loader, enable_async=True)
        self.env.filters["format_date"] = format_date

    async def close(self) -> None:
        await self.core.close()
        if self._own_client and self._client_entered:
            await self.client.__aexit__(None, None, None)
            self._client_entered = False

    async def _ensure_client(self) -> None:
        if self._own_client and not self._client_entered:
            await self.client.__aenter__()
            self._client_entered = True

    async def __aenter__(self) -> Renderer:
        await self._ensure_client()
        try:
            await self.core.launch()
        except Exception:
            await self.close()
            raise
        return self

    async def __aexit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: Any | None
    ) -> None:
        await self.close()

    @staticmethod
    async def _get_portrait(
        client: Client, data: str | ThreadDTO | BaseUserDTO, size: Literal["s", "m", "l"] = "s"
    ) -> bytes:
        """获取单个用户头像的二进制数据"""
        if isinstance(data, ThreadDTO):
            portrait = data.author.portrait
        elif isinstance(data, BaseUserDTO):
            portrait = data.portrait
        else:
            portrait = data

        if not portrait:
            return b""

        path = ""
        if size == "s":
            path = "n"
        elif size == "l":
            path = "h"

        img_url = yarl.URL.build(scheme="http", host="tb.himg.baidu.com", path=f"/sys/portrait{path}/item/{portrait}")
        try:
            response = await client.get_image_bytes(str(img_url))
        except Exception as e:
            logger.error(f"Failed to get portrait image from {img_url}: {e}")
            return b""

        return cast("bytes", response.data)

    @staticmethod
    async def _get_image(client: Client, image_hash: str, size: Literal["s", "m", "l"] = "s") -> bytes:
        """获取单张图片的二进制数据"""
        if not image_hash:
            return b""

        if size == "s":
            img_url = yarl.URL.build(
                scheme="http", host="imgsrc.baidu.com", path=f"/forum/w=720;q=60;g=0/sign=__/{image_hash}.jpg"
            )
        elif size == "m":
            img_url = yarl.URL.build(
                scheme="http", host="imgsrc.baidu.com", path=f"/forum/w=960;q=60;g=0/sign=__/{image_hash}.jpg"
            )
        elif size == "l":
            img_url = yarl.URL.build(scheme="http", host="imgsrc.baidu.com", path=f"/forum/pic/item/{image_hash}.jpg")
        else:
            raise ValueError("Size must be one of 's', 'm', or 'l'.")

        try:
            response = await client.get_image_bytes(str(img_url))
            return cast("bytes", response.data)
        except Exception as e:
            logger.error(f"Failed to get image from {img_url}: {e}")
            return b""

    @staticmethod
    async def _get_images(
        client: Client,
        data: ThreadDTO | PostDTO | list[str],
        size: Literal["s", "m", "l"] = "s",
        max_count: int | None = None,
    ) -> list[bytes]:
        """获取多张图片的二进制数据"""
        if isinstance(data, (ThreadDTO, PostDTO)):
            image_hash_list = [img.hash for img in data.images]
        else:
            image_hash_list = data

        if max_count is not None:
            image_hash_list = image_hash_list[:max_count]

        images_bytes = await asyncio.gather(*[
            Renderer._get_image(client, image_hash, size=size) for image_hash in image_hash_list
        ])

        return images_bytes

    @staticmethod
    async def _get_forum_icon(client: Client, fname: str) -> bytes:
        """根据贴吧名称获取吧头像"""
        if not fname:
            return b""

        try:
            forum_info = await client.get_forum(fname)
        except Exception as e:
            logger.error(f"Failed to get forum info for {fname}: {e}")
            return b""

        if not forum_info or not forum_info.small_avatar:
            return b""

        try:
            response = await client.get_image_bytes(forum_info.small_avatar)
            return cast("bytes", response.data)
        except Exception as e:
            logger.error(f"Failed to get forum icon image from {forum_info.small_avatar}: {e}")
            return b""

    @staticmethod
    def _get_mime_type(data: bytes) -> str:
        """根据二进制数据判断 MIME 类型"""
        if data.startswith(b"\xff\xd8\xff"):
            return "image/jpeg"
        if data.startswith(b"\x89PNG\r\n\x1a\n"):
            return "image/png"
        if data.startswith(b"GIF8"):
            return "image/gif"
        if data.startswith(b"RIFF") and data[8:12] == b"WEBP":
            return "image/webp"
        return "image/jpeg"

    def _bytes2base64_url(self, data: bytes) -> str:
        """将二进制数据转换为 Base64 编码的 URL"""
        if not data:
            return ""

        mime_type = self._get_mime_type(data)
        return f"data:{mime_type};base64,{base64.b64encode(data).decode('utf-8')}"

    async def _build_content_context(
        self,
        content: ThreadDTO | ThreadpDTO | PostDTO | CommentDTO,
        max_image_count: int = 9,
        show_link: bool = True,
    ) -> dict[str, Any]:
        """
        构建渲染内容上下文字典

        Args:
            content: 要构建上下文的内容，可以是 ThreadDTO、PostDTO 或 CommentDTO
            max_image_count: 最大包含的图片数量，默认为 9
            show_link: 是否显示 tid 和 pid，默认为 True

        Returns:
            dict[str, Any]: 包含渲染内容信息的字典
        """
        context: dict[str, Any] = {
            "text": content.text,
            "create_time": content.create_time,
            "nick_name": content.author.show_name or f"uid:{content.author.user_id}",
            "level": content.author.level,
            "portrait_url": "",
            "image_url_list": [],
            "remain_image_count": 0,
            "sub_text_list": [],
            "sub_html_list": [],
            "tid": content.tid,
            "pid": content.pid,
        }

        if isinstance(content, (ThreadDTO, ThreadpDTO, PostDTO)):
            context["image_hash_list"] = [img.hash for img in content.images]
        else:
            context["image_hash_list"] = []

        if isinstance(content, (ThreadDTO, ThreadpDTO)):
            context["title"] = content.title
            if show_link:
                context["sub_text_list"].append(f"tid: {content.tid}")
        elif isinstance(content, PostDTO):
            context["floor"] = content.floor
            if show_link:
                context["sub_text_list"].append(f"pid: {content.pid}")
            context["comments"] = [await self._build_content_context(c, max_image_count) for c in content.comments]
        elif isinstance(content, CommentDTO):
            pass

        portrait_bytes = b""
        if content.author.portrait:
            size: Literal["s", "m", "l"] = "s" if isinstance(content, CommentDTO) else "m"
            portrait_bytes = await self._get_portrait(self.client, content.author.portrait, size=size)

        images_bytes: list[bytes] = []
        if context["image_hash_list"]:
            images_bytes = await self._get_images(
                self.client, context["image_hash_list"], size="s", max_count=max_image_count
            )

        if portrait_bytes:
            context["portrait_url"] = self._bytes2base64_url(portrait_bytes)

        if images_bytes:
            context["image_url_list"] = [self._bytes2base64_url(img) for img in images_bytes]
            context["remain_image_count"] = max(0, len(context["image_hash_list"]) - len(context["image_url_list"]))

        return context

    async def _render_html(self, template_name: str, data: dict[str, Any]) -> str:
        """
        使用指定模板渲染 HTML

        Args:
            template_name: 模板名称
            data: 渲染数据字典

        Returns:
            str: 渲染后的 HTML 字符串
        """
        template = self.env.get_template(template_name)
        html = await template.render_async(**data)
        return html

    async def _render_image(
        self, template_name: str, config: RenderConfig | None = None, data: dict[str, Any] | None = None
    ) -> bytes:
        """
        使用指定模板渲染图像

        Args:
            template_name: 模板名称
            config: 渲染配置，若为 None 则使用默认配置
            data: 渲染数据字典

        Returns:
            bytes: 渲染后的图像字节数据
        """
        html = await self._render_html(template_name, data or {})
        image_bytes = await self.core.render(html, config or self.config)
        return image_bytes

    async def render_content(
        self,
        content: ThreadDTO | Thread | PostDTO | Post,
        *,
        max_image_count: int = 9,
        prefix_html: str | None = None,
        suffix_html: str | None = None,
        title: str = "",
        **config: Any,
    ) -> bytes:
        """
        渲染内容（贴子或回复）为图像

        Args:
            content: 要渲染的内容，可以是 Thread/Post 相关对象
            max_image_count: 最大包含的图片数量，默认为 9
            prefix_html: 文本前缀，可选，支持 HTML
            suffix_html: 文本后缀，可选，支持 HTML
            title: 覆盖标题，可选
            **config: 其他渲染配置参数

        Returns:
            生成的图像的字节数据
        """
        await self._ensure_client()

        render_config = self.config.model_copy(update=config)

        if isinstance(content, Thread):
            content = convert_aiotieba_thread(content)
        elif isinstance(content, Post):
            content = convert_aiotieba_post(content)

        content_context = await self._build_content_context(content, max_image_count)

        if title and isinstance(content, ThreadDTO):
            content_context["title"] = title

        if prefix_html:
            content_context["prefix_html"] = prefix_html
        if suffix_html:
            content_context["suffix_html"] = suffix_html

        forum_icon_url = ""
        if content.fname:
            icon_bytes = await self._get_forum_icon(self.client, content.fname)
            forum_icon_url = self._bytes2base64_url(icon_bytes)

        data = {
            "content": content_context,
            "forum": content.fname,
            "forum_icon_url": forum_icon_url,
            "prefix_html": prefix_html or "",
            "suffix_html": suffix_html or "",
            "style_list": [get_font_style()],
        }

        image_bytes = await self._render_image("thread.html", config=render_config, data=data)
        return image_bytes

    async def render_thread_detail(
        self,
        thread: ThreadDTO | ThreadpDTO | Thread | Thread_p,
        posts: Sequence[PostDTO | Post] | None = None,
        *,
        max_image_count: int = 9,
        prefix_html: str | None = None,
        suffix_html: str | None = None,
        ignore_first_floor: bool = True,
        show_thread_info: bool = True,
        show_link: bool = True,
        **config: Any,
    ) -> bytes:
        """
        渲染贴子详情（包含回复）为图像

        Args:
            thread: 要渲染的贴子
            posts: 要渲染的回复列表
            max_image_count: 每个楼层最大包含的图片数量，默认为 9
            prefix_html: 贴子文本前缀，可选，支持 HTML
            suffix_html: 贴子文本后缀，可选，支持 HTML
            ignore_first_floor: 是否忽略渲染第一楼（楼主），默认为 True
            show_thread_info: 是否显示贴子信息（转发、点赞、回复数），默认为 True
            show_link: 是否显示 tid 和 pid，默认为 True
            **config: 其他渲染配置参数

        Returns:
            生成的图像的字节数据
        """
        await self._ensure_client()
        render_config = self.config.model_copy(update=config)

        if isinstance(thread, Thread):
            thread = convert_aiotieba_thread(thread)
        elif isinstance(thread, Thread_p):
            thread = convert_aiotieba_threadp(thread)

        posts_dtos: list[PostDTO] = []
        if posts:
            for p in posts:
                if isinstance(p, Post):
                    posts_dtos.append(convert_aiotieba_post(p))
                else:
                    posts_dtos.append(p)

        if ignore_first_floor:
            posts_dtos = [p for p in posts_dtos if p.floor != 1]

        thread_context = await self._build_content_context(thread, max_image_count, show_link=show_link)
        posts_contexts = await asyncio.gather(*[
            self._build_content_context(p, max_image_count, show_link=show_link) for p in posts_dtos
        ])

        if show_thread_info:
            info_html = await self._render_html(
                "thread_info.html",
                {
                    "share_num": thread.share_num,
                    "agree_num": thread.agree_num,
                    "reply_num": thread.reply_num,
                },
            )
            thread_context["sub_html_list"].append(info_html)

        forum_icon_url = ""
        if thread.fname:
            icon_bytes = await self._get_forum_icon(self.client, thread.fname)
            forum_icon_url = self._bytes2base64_url(icon_bytes)

        data = {
            "thread": thread_context,
            "posts": posts_contexts,
            "forum": thread.fname,
            "forum_icon_url": forum_icon_url,
            "prefix_html": prefix_html or "",
            "suffix_html": suffix_html or "",
            "style_list": [get_font_style()],
        }

        image_bytes = await self._render_image("thread_detail.html", config=render_config, data=data)
        return image_bytes



from unittest.mock import AsyncMock, MagicMock, patch

import pytest

from tiebameow.models.dto import PostDTO, PostUserDTO, ThreadDTO, ThreadUserDTO
from tiebameow.renderer import Renderer
from tiebameow.renderer.playwright_core import PlaywrightCore

# --- Test PlaywrightCore ---


@pytest.mark.asyncio
async def test_playwright_core_launch():
    with patch("playwright.async_api.async_playwright") as mock_playwright:
        mock_playwright_context = AsyncMock()
        mock_playwright.return_value = mock_playwright_context

        mock_browser = AsyncMock()
        mock_playwright_context.start.return_value = mock_playwright_context
        mock_playwright_context.chromium.launch.return_value = mock_browser

        core = PlaywrightCore()
        await core.launch()

        assert core.playwright is not None
        assert core.browser is not None
        mock_playwright_context.chromium.launch.assert_called_once()


@pytest.mark.asyncio
async def test_playwright_core_close():
    core = PlaywrightCore()
    mock_browser = AsyncMock()
    mock_playwright = AsyncMock()
    core.browser = mock_browser
    core.playwright = mock_playwright

    await core.close()

    mock_browser.close.assert_called_once()
    mock_playwright.stop.assert_called_once()
    assert core.browser is None
    assert core.playwright is None


# --- Test Renderer ---


@pytest.fixture
def mock_playwright_core_cls():
    with patch("tiebameow.renderer.renderer.PlaywrightCore") as mock:
        yield mock


@pytest.fixture
def renderer(mock_playwright_core_cls):
    with patch("tiebameow.renderer.renderer.Client") as _:
        r = Renderer()
        # Mock the core instance
        r.core = AsyncMock(spec=PlaywrightCore)
        r.core.render = AsyncMock(return_value=b"image_bytes")
        r.core.close = AsyncMock()
        return r


@pytest.mark.asyncio
async def test_renderer_render_html(renderer):
    # Mock jinja2 environment and template
    mock_template = AsyncMock()
    mock_template.render_async.return_value = "<html></html>"
    renderer.env.get_template = MagicMock(return_value=mock_template)

    data = {"key": "value"}
    html = await renderer._render_html("test_template.html", data)

    assert html == "<html></html>"
    renderer.env.get_template.assert_called_with("test_template.html")
    mock_template.render_async.assert_called_with(**data)


@pytest.mark.asyncio
async def test_renderer_render_image(renderer):
    # Mock _render_html
    renderer._render_html = AsyncMock(return_value="<html></html>")

    image_bytes = await renderer._render_image("test_template.html")

    assert image_bytes == b"image_bytes"
    renderer._render_html.assert_called_once()
    renderer.core.render.assert_called_once()


@pytest.mark.asyncio
async def test_renderer_render_content(renderer):
    # Mock _render_image
    renderer._render_image = AsyncMock(return_value=b"content_image_bytes")
    # Mock _build_content_context to avoid network calls and complex logic
    renderer._build_content_context = AsyncMock(
        return_value={
            "text": "",
            "create_time": 0,
            "nick_name": "",
            "level": 0,
            "portrait_url": "",
            "image_url_list": [],
            "remain_image_count": 0,
            "sub_text_list": [],
            "sub_html_list": [],
            "tid": 0,
            "pid": 0,
            "image_hash_list": [],
        }
    )
    # Mock _get_forum_icon
    renderer._get_forum_icon = AsyncMock(return_value=b"")

    thread_dto = ThreadDTO.model_construct(
        tid=123,
        pid=456,
        fname="forum_name",
        author=ThreadUserDTO.model_construct(
            user_id=1, user_name="user", nick_name_new="nickname", portrait="portrait", level=1
        ),
        title="title",
        create_time=1234567890,
        posts=[],
        contents=[],
    )

    image_bytes = await renderer.render_content(thread_dto)

    assert image_bytes == b"content_image_bytes"
    renderer._render_image.assert_called_once()


@pytest.mark.asyncio
async def test_renderer_render_thread_detail(renderer):
    # Mock _render_image
    renderer._render_image = AsyncMock(return_value=b"thread_detail_image_bytes")
    # Mock _build_content_context
    renderer._build_content_context = AsyncMock(
        return_value={
            "text": "",
            "create_time": 0,
            "nick_name": "",
            "level": 0,
            "portrait_url": "",
            "image_url_list": [],
            "remain_image_count": 0,
            "sub_text_list": [],
            "sub_html_list": [],
            "tid": 0,
            "pid": 0,
            "image_hash_list": [],
        }
    )
    # Mock _get_forum_icon
    renderer._get_forum_icon = AsyncMock(return_value=b"")

    thread_dto = ThreadDTO.model_construct(
        tid=123,
        pid=456,
        fname="forum_name",
        author=ThreadUserDTO.model_construct(
            user_id=1, user_name="user", nick_name_new="nickname", portrait="portrait", level=1
        ),
        title="title",
        create_time=1234567890,
        posts=[],
        contents=[],
        share_num=0,
        agree_num=0,
        reply_num=0,
    )

    post_dto = PostDTO.model_construct(
        pid=789,
        tid=123,
        floor=2,
        author=PostUserDTO.model_construct(
            user_id=2, user_name="user2", nick_name_new="nickname2", portrait="portrait", level=1
        ),
        contents=[],
        create_time=1234567890,
    )

    image_bytes = await renderer.render_thread_detail(thread_dto, [post_dto])

    assert image_bytes == b"thread_detail_image_bytes"
    renderer._render_image.assert_called_once()


@pytest.mark.asyncio
async def test_get_portrait():
    mock_client = MagicMock()
    mock_response = MagicMock()
    mock_response.data = b"portrait_bytes"
    mock_client.get_image_bytes = AsyncMock(return_value=mock_response)

    # Test with string
    res = await Renderer._get_portrait(mock_client, "portrait_id")
    assert res == b"portrait_bytes"
    mock_client.get_image_bytes.assert_called_once()


@pytest.mark.asyncio
async def test_get_images():
    mock_client = MagicMock()
    mock_response = MagicMock()
    mock_response.data = b"img_bytes"
    mock_client.get_image_bytes = AsyncMock(return_value=mock_response)

    res = await Renderer._get_images(mock_client, ["hash1", "hash2"])
    assert res == [b"img_bytes", b"img_bytes"]
    assert mock_client.get_image_bytes.call_count == 2


@pytest.mark.asyncio
async def test_get_forum_icon():
    mock_client = MagicMock()
    mock_forum = MagicMock()
    mock_forum.small_avatar = "http://avatar.url"
    mock_client.get_forum = AsyncMock(return_value=mock_forum)

    mock_response = MagicMock()
    mock_response.data = b"icon_bytes"
    mock_client.get_image_bytes = AsyncMock(return_value=mock_response)

    res = await Renderer._get_forum_icon(mock_client, "forum_name")
    assert res == b"icon_bytes"
    mock_client.get_forum.assert_called_with("forum_name")
    mock_client.get_image_bytes.assert_called_with("http://avatar.url")



from __future__ import annotations

from asyncio import Lock
from typing import TYPE_CHECKING, Any, Literal, cast

if TYPE_CHECKING:
    from playwright.async_api import Browser, Playwright

    from .config import RenderConfig


QUALITY_MAP_SCALE = {
    "low": 1,
    "medium": 1.5,
    "high": 2,
}
QUALITY_MAP_OUTPUT: dict[str, dict[str, Any]] = {
    "low": {
        "type": "jpeg",
        "quality": 60,
    },
    "medium": {
        "type": "jpeg",
        "quality": 80,
    },
    "high": {"type": "png"},
}

VALID_BROWSER_ENGINES = Literal["chromium", "firefox", "webkit"]


class PlaywrightCore:
    def __init__(self, browser_engine: VALID_BROWSER_ENGINES | None = None) -> None:
        if not self.check_installed():
            raise ImportError(
                "playwright is not installed. Please install it with 'pip install playwright'.\n"
                "You may also need to run 'playwright install' to install the necessary browsers."
            )

        self.playwright: Playwright | None = None
        self.browser_engine = browser_engine or "chromium"
        self.browser: Browser | None = None
        self._lock = Lock()

    @staticmethod
    def check_installed() -> bool:
        """检查playwright包是否已安装。"""
        try:
            import playwright  # noqa: F401
        except ImportError:
            return False
        return True

    async def launch(self) -> None:
        async with self._lock:
            if self.browser is not None:
                return

            if self.playwright is None:
                from playwright.async_api import async_playwright

                self.playwright = await async_playwright().start()

            engine = getattr(self.playwright, self.browser_engine)
            if not engine:
                raise ValueError(f"Invalid browser engine: {self.browser_engine}")
            try:
                self.browser = await engine.launch()
            except AttributeError as e:
                raise ValueError(f"Invalid browser engine: {self.browser_engine}") from e

    async def close(self) -> None:
        async with self._lock:
            if self.browser is not None:
                await self.browser.close()
                self.browser = None
            if self.playwright is not None:
                await self.playwright.stop()
                self.playwright = None

    async def render(self, html: str, config: RenderConfig) -> bytes:
        async with self._lock:
            if self.browser is None:
                await self.launch()

        browser = cast("Browser", self.browser)
        async with await browser.new_page(device_scale_factor=QUALITY_MAP_SCALE[config.quality]) as page:
            await page.set_viewport_size({"width": config.width, "height": config.height})

            await page.set_content(html)
            await page.wait_for_load_state("networkidle")
            screenshot = await page.screenshot(full_page=True, **QUALITY_MAP_OUTPUT[config.quality])
            return screenshot



import base64
from functools import lru_cache
from pathlib import Path

font_path = Path(__file__).parent / "static" / "fonts" / "NotoSansSC-Regular.woff2"


@lru_cache(maxsize=1)
def get_font_style() -> str:
    if not font_path.exists():
        return """<style>
body {
    font-family: "Noto Sans SC", "Noto Sans CJK SC", sans-serif;
    font-size: 14px;
}
</style>
"""

    with font_path.open("rb") as f:
        font_data = base64.b64encode(f.read()).decode("utf-8")
    return f"""<style>
@font-face {{
    font-family: 'Noto Sans SC';
    font-style: normal;
    font-weight: 400;
    src: url(data:font/woff2;base64,{font_data}) format('woff2');
}}

body {{
    font-family: "Noto Sans SC", "Noto Sans CJK SC", sans-serif;
    font-size: 14px;
}}
</style>
"""



from datetime import datetime
from typing import Literal

from pydantic import BaseModel

from tiebameow.models.dto import (
    BaseDTO,
    BaseUserDTO,
    ShareThreadDTO,
    ThreadDTO,
    ThreadUserDTO,
)
from tiebameow.schemas.fragments import FragTextModel

# --- Test Helpers ---


class SimpleDTO(BaseDTO):
    name: str
    age: int
    score: float
    is_active: bool
    tags: list[str]
    metadata: dict[str, str]


class NestedDTO(BaseDTO):
    id: int
    child: SimpleDTO


class LiteralDTO(BaseDTO):
    mode: Literal["A", "B", "C"]


class PlainModel(BaseModel):
    x: int
    y: int


class WrapperDTO(BaseDTO):
    inner: PlainModel


# --- Normal Tests ---


def test_base_user_dto() -> None:
    user = BaseUserDTO(user_id=1, portrait="portrait", user_name="user_name", nick_name_new="nick_name")
    assert user.nick_name == "nick_name"
    assert user.show_name == "nick_name"

    user_no_nick = BaseUserDTO(user_id=1, portrait="portrait", user_name="user_name", nick_name_new="")
    assert user_no_nick.nick_name == ""
    assert user_no_nick.show_name == "user_name"


def test_thread_user_dto() -> None:
    user = ThreadUserDTO(
        user_id=1,
        portrait="portrait",
        user_name="user_name",
        nick_name_new="nick_name",
        level=1,
        glevel=1,
        gender="MALE",
        icons=[],
        is_bawu=False,
        is_vip=False,
        is_god=False,
        priv_like="PUBLIC",
        priv_reply="ALL",
    )
    assert user.level == 1
    assert user.gender == "MALE"


def test_thread_dto() -> None:
    author = ThreadUserDTO(
        user_id=1,
        portrait="portrait",
        user_name="user_name",
        nick_name_new="nick_name",
        level=1,
        glevel=1,
        gender="MALE",
        icons=[],
        is_bawu=False,
        is_vip=False,
        is_god=False,
        priv_like="PUBLIC",
        priv_reply="ALL",
    )
    share_origin = ShareThreadDTO(pid=0, tid=0, fid=0, fname="", author_id=0, title="", contents=[])
    thread = ThreadDTO(
        pid=1,
        tid=1,
        fid=1,
        fname="fname",
        author_id=1,
        author=author,
        title="title",
        contents=[FragTextModel(text="content")],
        is_good=False,
        is_top=False,
        is_share=False,
        is_hide=False,
        is_livepost=False,
        is_help=False,
        agree_num=0,
        disagree_num=0,
        reply_num=0,
        view_num=0,
        share_num=0,
        create_time=datetime.now(),
        last_time=datetime.now(),
        thread_type=0,
        tab_id=0,
        share_origin=share_origin,
    )
    assert thread.title == "title"
    assert len(thread.contents) == 1
    assert isinstance(thread.contents[0], FragTextModel)
    assert thread.contents[0].text == "content"


# --- Zero-fill Tests ---


def test_base_dto_zero_values() -> None:
    """Test completely empty init fills with zero values."""
    obj = SimpleDTO.from_incomplete_data({})
    assert obj.name == ""
    assert obj.age == 0
    assert obj.score == 0.0
    assert obj.is_active is False
    assert obj.tags == []
    assert obj.metadata == {}

    # Test with None input
    obj_none = SimpleDTO.from_incomplete_data(None)
    assert obj_none.name == ""


def test_base_dto_partial_values() -> None:
    """Test partial data fills missing with zero values."""
    obj = SimpleDTO.from_incomplete_data({"name": "Alice", "age": 30})
    assert obj.name == "Alice"
    assert obj.age == 30
    assert obj.score == 0.0  # Filled
    assert obj.tags == []  # Filled


def test_base_dto_nested() -> None:
    """Test recursion on nested BaseDTO fields."""
    obj = NestedDTO.from_incomplete_data({"id": 1, "child": {"name": "Bob"}})
    assert obj.id == 1
    assert isinstance(obj.child, SimpleDTO)
    assert obj.child.name == "Bob"
    assert obj.child.age == 0  # Filled child field
    assert obj.child.tags == []


def test_base_dto_nested_empty() -> None:
    """Test recursion when nested field is missing entirely."""
    obj = NestedDTO.from_incomplete_data({"id": 1})
    assert obj.id == 1
    assert isinstance(obj.child, SimpleDTO)
    # Child should be created with all zero values
    assert obj.child.name == ""
    assert obj.child.age == 0


def test_base_dto_literal() -> None:
    """Test Literal defaults to first option."""
    obj = LiteralDTO.from_incomplete_data({})
    assert obj.mode == "A"


def test_base_dto_literal_provided() -> None:
    """Test Literal with provided value."""
    obj = LiteralDTO.from_incomplete_data({"mode": "B"})
    assert obj.mode == "B"


def test_base_dto_with_plain_pydantic_model() -> None:
    """Test nesting a plain Pydantic model (not BaseDTO) handles partial filling."""
    # Only partial data for inner model
    obj = WrapperDTO.from_incomplete_data({"inner": {"x": 10}})

    # _get_zero_value for PlainModel should create {x:0, y:0}
    # Then merged with {x: 10} -> {x:10, y:0}
    assert isinstance(obj.inner, PlainModel)
    assert obj.inner.x == 10
    assert obj.inner.y == 0
